(* file: fibo.ml
 * author: Bob Muller
 *
 * Two variations of a function for retrieving the Nth Fibonacci
 * number. The Fibonacci sequence is:
 *
 *     1, 1, 2, 3, 5, 8, 13, ... F_(n-1) + F_(n-2)
 *
 * The first "solution" isn't a reasonable solution because it
 * requires an exponential amount of work. 
 *
 * badFibo : int -> int        this solution does too much work!
 *)
let rec badFibo n =
  match n < 3 with
  | true  -> 1
  | false -> (badFibo (n - 1)) + (badFibo (n - 2))

(* goodFibo : int -> int       this solution does less work!
 *)
let fastFibo n =
  let rec repeat n a b =
    match n = 1 with
    | true  -> a
    | false -> repeat (n - 1) b (a + b)
  in
  repeat n 1 1
